`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc 2011
// Engineer: Michelle Yu  
// Create Date:    14:11:54 08/25/2011
// Module Name:    PmodPS2
// Project Name: 	 PmodPS2_Demo
// Target Devices: Nexys 3 
// Tool versions:  ISE 14.2
//	Description: This project uses reference component MouseRefComp, VgaRefComp and
//	Mouse_displayer to display a mouse cusor on a vga monitor. The mouse cursor is
//	controlled by a PS/2 mouse connected via PmodPS2 onto the lower row of connector JA.
//	SW0 on the Nexys3 board is used to control the resolution, when the switch is on the
// resolution is 800x600, when the switch is off the resolution is 640x480. BTNR is used
//	as reset. LED0 turns on when right click on the mouse. LED1 turns on when middle click
//	on the mouse. LED2 turns on when left click on the mouse.
// 
// Revision:
// Revision 0.01 - File Created
// Revision 1.00 - Converted from VHDL to Verilog (Josh Sackos)
//////////////////////////////////////////////////////////////////////////////////////////

// =======================================================================================
// 										  Define Module
// =======================================================================================
module PmodPS2(
		clk,
		sw,
		btnr,
		btnl,
		LED,
		JA,
		vgaRed,
		vgaGreen,
		vgaBlue,
		Hsync,
		Vsync
);

// =======================================================================================
// 										Port Declarations
// =======================================================================================
   input        clk;		// CLK
   input [1:0]  sw;		// sw0 RESOLUTION
   input        btnr;	// btnr RST
   input        btnl;	// btnl refresh after resolution
   output [2:0] LED;		// signal from the mouse
								// LED2 LEFT
								// LED1 MIDDLE
								// LED0 RIGHT

   // signal from the pmod
   inout [7:4]  JA;
   
   // color control 
   output [2:0] vgaRed;
   output [2:0] vgaGreen;
   output [2:1] vgaBlue;
   output       Hsync;
   output       Vsync;
   
   
// =======================================================================================
// 							  Parameters, Registers, and Wires
// =======================================================================================

			wire [9:0]   X;
			wire [9:0]   Y;
			wire [10:0]  HC;
			wire [10:0]  VC;
			wire         BLK;
			wire         CLK25;
			wire         CLK40;
			wire         CLKpix;
			wire         CLKSYS;
			wire         NewEvent;


// =======================================================================================
// 							  		   Implementation
// =======================================================================================

	//--------------------------------------
	//	  use BUFGMUX for clock selection
	//--------------------------------------
   BUFGMUX BUFGMUX_pix(
			.O(CLKpix),
			.I0(CLK25),
			.I1(CLK40),
			.S(sw[0])
	);
   

	//--------------------------------------
	//			Mouse Reference Component
	//--------------------------------------
   MouseRefComp C0(
				.CLK(CLKpix),
				.RESOLUTION(sw[0]),
				.SWITCH(btnl),
				.RST(btnr),
				.LEFT(LED[2]),
				.MIDDLE(LED[1]),
				.NEW_EVENT(),
				.RIGHT(LED[0]),
				.XPOS(X),
				.YPOS(Y),
				.ZPOS(),
				.PS2_CLK(JA[6]),
				.PS2_DATA(JA[4])
	);
   
   
	//--------------------------------------
	//				 Clock Signals
	//--------------------------------------
   dcm_all C1(
				.CLK(clk),
				.RST(btnr),
				.CLKSYS(CLKSYS),
				.CLK25(CLK25),
				.CLK40(CLK40)
	);
   
   
	//--------------------------------------
	//			 VGA Monitor Interface
	//--------------------------------------
   VgaRefComp C2(
				.CLK_25MHz(CLK25),
				.CLK_40MHz(CLK40),
				.RESOLUTION(sw[0]),
				.RST(btnr),
				.BLANK(BLK),
				.HCOUNT(HC),
				.HS(Hsync),
				.VCOUNT(VC),
				.VS(Vsync)
	);
   
	//--------------------------------------
	//	Set background to zero by setting
	// red_in, green_in and blue_in to zero
	//--------------------------------------
   mouse_displayer C3(
				.clk(CLKSYS),
				.pixel_clk(CLKpix),
				.xpos(X),
				.ypos(Y),
				.hcount(HC),
				.vcount(VC),
				.blank(BLK),
				.red_in(3'b000),
				.green_in(3'b000),
				.blue_in(2'b00),
				.red_out(vgaRed),
				.green_out(vgaGreen),
				.blue_out(vgaBlue)
	);
   
endmodule
